
	Title	Audio Port I/O routines
	page	63,131

;
;-------------------====< Audio Port I/O Routines >====----------------------
;
; Copyright (c) 1992, Media Vision, Inc. All Rights Reserved
;

        .model  small,c
	.286

        include masm.inc
	include parr.inc

Idledly   equ   <out dx,al>

        .data

	public	PortAddress
PortAddress     dw      378h

	.code

;
;   /*\
;---|*|----====< AP_ADC_input >====----
;---|*|
;---|*| Input 256 bytes from the AP FIFO. This routine
;---|*| assumes that the AP is already in ADC input mode.
;---|*|
;---|*| Entry Conditions:
;---|*|   ES:DI point to the buffer.
;---|*|   CX holds a count of 1 to 256
;---|*|
;---|*| Exit Conditions:
;---|*|   ES:DI is updated. CX is zero
;---|*|   AX,BX,CX,DX,ES:DI modified
;---|*|
;   \*/
;
	public	AP_ADC_input
AP_ADC_input	proc	near
;
; load the FIFO quarter size for use with the SRQ handshaking.
;
        mov     dx,[PortAddress]        ; get the port address
	add	dl,_CONTROL
	cld				; STOSBs are used
    ;
    apadc05:

        mov     al,RdIdle               ; go into Read Idle to pick up the nibble
        out     dx,al                   ; Idle w/o interrupts
	Idledly

	dec	dx			; move to STATUS
	in	al,dx
	mov	ah,al			; save the high nibble
	inc	dx

	mov	al,RdSndData		; ah = RdIdle, al = RdSndData
	out	dx,al			; this delivers the second nibble
	Idledly

	dec	dx
	in	al,dx			; fetch it

	shr	ah,4			; merge the nibbles
	shr	ax,4

	stosb				; save the byte
	or	di,di			; segment wrap?
	jz	apadcsegwrap		; yes, go adjust...
    ;
    apadc10:
	inc	dx			; place the board into idle mode again
	loop	apadc05 		; do all 256 or less bytes

	mov	al,WrIdle		; return back to write idle
	out	dx,al
        Idledly

	ret				; all done recording this block!
;
apadcsegwrap:
	mov	di,es			; wrap the segment for huge model
	add	di,1000h		; support.
	mov	es,di
	sub	di,di
	jmp	short  apadc10

AP_ADC_input	endp

;
;   /*\
;---|*|----====< AP_DAC_output >====----
;---|*|
;---|*| Output 256 bytes to the AP FIFO. This routine
;---|*| assumes that the AP is already in DAC output mode.
;---|*|
;---|*| Entry Conditions:
;---|*|   ES:DI point to the buffer.
;---|*|   CX holds a count of 1 to 256
;---|*|
;---|*| Exit Conditions:
;---|*|   ES:DI is updated. CX is zero
;---|*|   AX,CX,DX,ES:DI modified
;---|*|
;   \*/
;
	public	AP_DAC_output
AP_DAC_output	proc	near
;
; make sure the port is in write idle mode...
;
        mov     dx,[PortAddress]        ; get the port address
	add	dl,_CONTROL
	mov	al,WrIdle		; make sure its idle
	out	dx,al
	sub	dl,_CONTROL		; move back to the data port
;
apdac05:
;
; send 256 bytes of data out to the FIFO
;
	mov	al,es:[di]		; fetch the byte
	inc	di
	jz	apdacsegwrap		; odd case jumps out...
    ;
    apdac10:

        out     dx,al                   ; write it to the audio port

        add     dl,_CONTROL

	mov	al,WrSndData		; start the strobe...
	out	dx,al

	mov	al,WrIdle		; end the strobe....
	out	dx,al
	Idledly

	sub	dl,_CONTROL
	loop	apdac05 		; do all 256 or less bytes

	ret
;
apdacsegwrap:
	mov	di,es			; move to the next segment
	add	di,1000h
	mov	es,di
	sub	di,di
	jmp	short apdac10		; then get back into the playback

AP_DAC_output   endp

;
;   /*\
;---|*|----====< AP_Reset >====----
;---|*|
;---|*| Reset the Audio Port
;---|*|
;---|*| Entry Conditions:
;---|*|   None.
;---|*|
;---|*| Exit Conditions:
;---|*|   AL holds the reset byte - 05A if successful
;---|*|   AX,BX,CX,DX modified
;---|*|
;   \*/
;
	public	AP_Reset
AP_Reset proc

	mov	dx,[PortAddress]
	mov	al,-1		; load FF into the data port to drive the
	out	dx,al		; data lines.

        add     dl,_CONTROL
	mov	cx,16
	mov	al,WrReset2	; output reset mode
;
@@:
        out     dx,al
	loop	@B

        mov     cx,16
;
@@:
        mov     al,WrIdle       ; clear reset
	out	dx,al
        loop    @B

	mov	al,RdIdle	; get sound data read status
	out	dx,al
	Idledly

	dec	dx		; move back to the status port
	sub	cx,cx
;
aprst05:
	in	al,dx
	test	al,stIDav	; wait for data to become available
	loopz	aprst05
	jz	aprstbad

	mov	ah,al		; save the result (should not be 0xFF)

	inc	dx		; move to the control port
	mov	al,RdSndData	; Clock in the data
	out	dx,al

	dec	dx
	in	al,dx
	mov	bx,ax

	inc	dx
	mov	al,RdIdle	; go back to idle
	out	dx,al
	Idledly

	mov	al,WrIdle	; go back to idle
	out	dx,al
	Idledly

	mov	cl,4		; build the reset byte.
	shr	bh,cl
	shr	bx,cl

	mov	al,bl
	ret
    ;
    aprstbad:
	mov	al,-1
	ret

AP_reset endp

;
;   /*\
;---|*|----====< AP_MDR_stat >====----
;---|*|
;---|*| Returns the status of data available
;---|*|
;---|*| Entry Conditions:
;---|*|   None.
;---|*|
;---|*| Exit Conditions:
;---|*|   Carry clear & AX = 0,  no data available
;---|*|   Carry set   & AX = -1, data available
;---|*|   DX,AX modified
;---|*|
;   \*/
;
	public	AP_MDR_stat
AP_MDR_stat	proc
	mov	dx,[PortAddress]
	add	dl,_CONTROL

	mov	al,RdIdle	; Micro read idle
	out	dx,al
	Idledly

	dec	dx		; mov to the status port

	in	al,dx		; data rdy is high for data available
	mov	ah,stIDav
	and	ah,al		; save the status bit

	mov	al,WrIdle	; send the port back to
	out	dx,al		; idle mode
	Idledly

	neg	ah		; set carry if non-zero
	sbb	ax,ax		; if data is available, return -1 & Cy
	ret			; else return 0 & NC

AP_MDR_stat	endp

;
;   /*\
;---|*|----====< AP_SRQ_stat >====----
;---|*|
;---|*| Returns the status of Service Request
;---|*|
;---|*| Entry Conditions:
;---|*|   None.
;---|*|
;---|*| Exit Conditions:
;---|*|   Carry clear & AX = 0,  wait
;---|*|   Carry set   & AX = -1, service request is posted
;---|*|   DX,AX modified
;   \*/
;
        public  AP_SRQ_stat
AP_SRQ_stat	proc

	mov	dx,[PortAddress]
	add	dl,_CONTROL
	mov	al,WrIdle	; Write Idle mode...
	out	dx,al
	Idledly

	dec	dx		; mov to the status port

	in	al,dx		; data rdy is high for data available
	and	al,stSRQ

	neg	al		; set carry if non-zero
	sbb	ax,ax		; if data is available, return -1 & Cy
	ret			; else return 0 & NC

AP_SRQ_stat	endp

;
;   /*\
;---|*|----====< AP_MDR_wait >====----
;---|*|
;---|*| Wait for Micro Data Read to be ready.
;---|*|
;---|*| Entry Conditions:
;---|*|   None.
;---|*|
;---|*| Exit Conditions:
;---|*|   Carry clear & AX = 0,  no data available
;---|*|   Carry set   & AX = -1, data available
;---|*|   DX,AX modified
;---|*|
;   \*/
;
	public	AP_MDR_wait
AP_MDR_wait	proc
	push	cx

	sub	cx,cx
;
@@:
	call	AP_MDR_stat
	or	ax,ax
	loopz	@B

	pop	cx
	ret

AP_MDR_wait	endp

;
;   /*\
;---|*|----====< AP_MCW >====----
;---|*|
;---|*| Send a command byte out to the Micro.
;---|*|
;---|*| Entry Conditions:
;---|*|    AL holds the command
;---|*|
;---|*| Exit Conditions:
;---|*|    AX = -1 if data is available, or 0
;---|*|    DX,AX modified
;---|*|
;   \*/
;
	public	AP_MCW
AP_MCW  proc
	push	cx

        mov     ah,al           ; save the command/data

;
apmcw_00:
	mov	dx,[PortAddress]
	add	dl,_CONTROL
	mov	al,WrIdle
	out	dx,al
	Idledly

	mov	al,ah		; get the byte and write it out
	sub	dl,_CONTROL	; move to the data port
	out	dx,al		; load the data

        inc     dx              ; move to the status port
	sub	cx,cx
;
apmcw_05:
	in	al,dx		; wait=1 when cmd/data cannot be sent
	test	al,stWait
	loopnz	apmcw_05
	jz	apmcw_10	; its available

	push	ax		; The device is probably powered off, so
	call	AP_reset	; we just have to kick it...
	pop	ax		; AH holds the command/data byte
	jmp	short apmcw_00
;
apmcw_10:
	inc	dx		; move to the control port

	mov	al,WrSndCmd	; start the strobe
	out	dx,al

	mov	al,WrIdle	; end the strobe
	out	dx,al
	Idledly

	pop	cx
	ret

AP_MCW	endp

;
;   /*\
;---|*|----====< AP_MDW >====----
;---|*|
;---|*| Send a data byte out to the FIFO.
;---|*|
;---|*| Entry Conditions:
;---|*|    AL holds the data
;---|*|
;---|*| Exit Conditions:
;---|*|    AX = -1 if data is available, or 0
;---|*|    DX,AX modified
;---|*|
;   \*/
;
	public	AP_MDW
AP_MDW  proc
	push	cx

	mov	ah,al		; save the command/data
;
apmdw_00:
	mov	dx,[PortAddress]
	add	dl,_CONTROL

	mov	al,WrIdle	; make sure its' idle
	out	dx,al
	Idledly

	mov	al,ah		; get the byte and write it out
	sub	dl,_CONTROL	; move to the data port
	out	dx,al		; load the data

	inc	dx		; move to the status port
	sub	cx,cx
;
apmdw_05:
	in	al,dx		; wait=1 when cmd/data cannot be sent
	test	al,stWait
	loopnz	apmdw_05
	jz	apmdw_10	; its available

	push	ax		; The device is probably powered off, so
	call	AP_reset	; we just have to kick it...
	pop	ax		; AH holds the command/data byte
	jmp	short apmdw_00
;
apmdw_10:
	inc	dx		; move to the control port

	mov	al,WrSndData	; start the strobe
	out	dx,al

	mov	al,WrIdle	; end the strobe
	out	dx,al
	Idledly

	pop	cx
	ret

AP_MDW	endp

;
;   /*\
;---|*|----====< AP_MDR >====----
;---|*|
;---|*| Read a data byte from the Micro of FIFO
;---|*|
;---|*| Entry Conditions:
;---|*|    Data Available must have been checked before calling
;---|*|    this routine. It is assumed to be high.
;---|*|
;---|*| Exit Conditions:
;---|*|    AL holds something...
;---|*|    DX,AX modified
;---|*|
;   \*/
;
	public	AP_MDR
AP_MDR	proc
	push	cx

	mov	dx,[PortAddress]
	add	dl,_CONTROL

	mov	al,RdIdle	; go to read IDLE
	out	dx,al
	Idledly

	mov	al,RdSndData	; read the high nibble
	out	dx,al		; start the strobe

	dec	dx		; grab one nibble
	in	al,dx		; from the status port
	mov	ah,al		; save in AH!

	inc	dx
	mov	al,RdIdle	; swap nibbles
	out	dx,al		; end the strobe
	Idledly

	dec	dx
	in	al,dx		; read the next nibble

	mov	cl,WrIdle
	xchg	ax,cx		; save the bytes, load WrIdle

	inc	dx		; send H/W into write idle mode
	out	dx,al

	xchg	ax,cx		; get the bytes

        mov     cl,4
	shr	ah,cl		; this order makes for a
	shr	ax,cl		; fast conversion

        pop     cx
	ret

AP_MDR	endp

;
;   /*\
;---|*|----====< AP_FM_out >====----
;---|*|
;---|*| Send index/data out to the parallel FM chip
;---|*|
;---|*| Entry Conditions:
;---|*|    AL holds the index
;---|*|    AH holds the data
;---|*|
;---|*| Exit Conditions:
;---|*|    None
;---|*|    DX,AX modified
;   \*/
;
	public	AP_FM_out
AP_FM_out       proc
	push	cx
	push	ax		; save the index/data
;
apfmo00:
        mov     dx,[PortAddress]
	add	dl,_CONTROL	; move to the status port

	mov	al,WrFMIdle	; go into FM Mode
	out	dx,al
	Idledly

	dec	dx		; move back to status
        sub     cx,cx           ; loop control
;
apfmo05:
	in	al,dx
	test	al,stWait	; FM wait high?
	loopnz	apfmo05 	; yes, dont write it
	jz	apfmo10 	; its available

	call	AP_reset	; The device is probably powered off, so
	jmp	short apfmo00	; we just have to kick it...
;
apfmo10:

	pop	ax		; get the index and data
	jnz	apfmobad	; bomb out if we cant do it

	dec	dx		; move to the data port
	out	dx,al		; load the data port

	inc	dx		; move to the control register
	inc	dx

        mov     al,WrFMAddr
	out	dx,al		; start the strobe

	mov	al,WrFMIdle
	out	dx,al		; end the strobe
	Idledly

	mov	al,ah
	sub	dl,_CONTROL	; move to the data port
	out	dx,al		; load the data port

	add	dl,_CONTROL
	mov	al,WrFMData
	out	dx,al		; start the strobe

	mov	al,WrFMIdle
	out	dx,al		; end the strobe
	Idledly

	mov	al,WrIdle
	out	dx,al		; back to write idle...

;
apfmobad:
	pop	cx
        ret

AP_FM_out	endp

;
;   /*\
;---|*|----====< AP_FM_in >====----
;---|*|
;---|*| Read the FM status register
;---|*|
;---|*| Entry Conditions:
;---|*|    None
;---|*|
;---|*| Exit Conditions:
;---|*|    AL holds the status byte
;---|*|    DX,AX modified
;   \*/
;
	public	AP_FM_in
AP_FM_in        proc
	mov	dx,[PortAddress]
	add	dl,_CONTROL	; point to control port

	mov	al,WrIdle	; make sure we're idle
	out	dx,al
	Idledly

	mov	al,RdFMIdle	; go into FM Mode
	out	dx,al
	Idledly

	mov	al,RdFMStat	; clock in the data
	out	dx,al

	dec	dx		; move to the status port
	in	al,dx
	mov	bl,al		; save the high nibble

	inc	dx
	mov	al,RdFMIdle	; all set
	out	dx,al		; strobe address
	Idledly

	dec	dx
	in	al,dx		; clock in the data
	mov	bh,al

	inc	dx
	mov	al,WrIdle	; go back to idle mode
	out	dx,al
	Idledly

	mov	cl,4
	shr	bh,cl
	shr	bx,cl

	mov	al,bl

	ret

AP_FM_in	endp

	end

