
;;
;;   /*\
;;---|*|----====< Parallel Port Audio Adapter >====----
;;---|*|----====<     Copyright 1991,1992.    >====----
;;---|*|----====<      Media Vision, Inc.     >====----
;;---|*|----====<     All Rights Reserved     >====----
;;   \*/
;;
;;
;; This file contains the bit definitions for the PC side of the parallel port
;; The parallel port has three registers defined for it's use:
;;
;;	NAME		 LPT1  LPT2  MONO  DESCRIPTION
;;     Ŀ
;;     Data Output R  378  278  3BC write to TP	   
;;     Ĵ
;;     Status	    R  379  279  3BD read status/data 
;;     Ĵ
;;     Control     W  37A  27A  3BE set I/O control  
;;     
;;
;;
;; Parallel port offsets from the base I/O address
;;

_DATAPORT	equ	0
_STATUS 	equ	1
_CONTROL	equ	2

;;
;;
;; Control Register Definitions
;;
;;		   D7	D6   D5   D4   D3   D2	 D1   D0
;;     Ŀ
;;     Control  PAR PAR PAR IRQ  AS MFM  DS  RW 
;;     
;;		   |	|    |	  |    |    |	 |    |
;;		   |	|    |	  |    |    |	 |    +---- Read(0)/Write(1)
;;		   |	|    |	  |    |    |	 +--------- Data Strobe
;;		   |	|    |	  |    |    +-------------- Micro/FM select
;;		   |	|    |	  |    +------------------- Address Strobe
;;		   |	|    |	  +------------------------ IRQ Enable
;;		   +----+----+----------------------------- Parallel input select
;;
;;

RdFMIdle	equ	00h	;; 0000B Read FM Idle
WrFMIdle	equ	01h	;; 0001B Write FM Idle
invalid1	equ	02h	;; 0010B invalid state
WrFMData	equ	03h	;; 0011B Write FM Data
RdIdle		equ	04h	;; 0100B Read Sound Idle
WrIdle		equ	05h	;; 0101B Write Sound Idle
RdSndData	equ	06h	;; 0110B Read Sound Data
WrSndData	equ	07h	;; 0111B Write Sound Data
RdFMStat	equ	08h	;; 1000B Get FM Status
WrFMAddr	equ	09h	;; 1001B Write FM Address
invalid2	equ	0Ah	;; 1010B invalid state
invalid4	equ	0Bh	;; 1011B invalid state
RdSndStat	equ	0Ch	;; 1100B Read Sound Status
WrSndCmd	equ	0Dh	;; 1101B Write Sound Command
invalid3	equ	0Eh	;; 1110B invalid state
WrReset2	equ	0Fh	;; 1111B Write Reset

Interrupt	equ	10h	;;10000B Enable interrupt
EnableInput	equ	0E0h	;;	 Enable Parallel Input

DStrb		equ	02h	;; 0010B Data Strobe
CStrb		equ	08h	;; 1000B Command Strobe


;;
;; Status Register Definitions
;;
;;		   D7	D6   D5   D4   D3   D2	 D1   D0
;;     Ŀ
;;     Status	  W  IRQ PAR SRQ  DA  X   X   X  
;;     
;;		   |	|    |	  |    |    |	 |    |
;;		   |	|    |	  |    |    +----+----+---- unused
;;		   |	|    |	  |    +------------------- Data Available
;;		   |	|    |	  +------------------------  SRQ/D4/D0
;;		   |	|    +-----------------------------  PAR/D5/D1
;;		   |	+----------------------------------  IRQ/D6/D2
;;		   +--------------------------------------- WAIT/D7/D3
;;
;;

stIDav		equ	08h	;; Input Data Available
stSRQ		equ	10h	;; Service Request
stPARInp	equ	20h	;; Parallel Input Enable
stIRQ		equ	40h	;; Interrupt Request Status
stWait		equ	80h	;; M/FM Register Wait status

;;
;; Command Set:
;;
;;   Ŀ
;;   	  Command     D7-D4 Code   D3       D2    	 D1       D0	 
;;   Ĵ
;;   	 reserved         0	     X    	X    	  X        X	 
;;   Ĵ
;;   System Control      1	 		  Subcode		 
;;   Ĵ
;;   8 Bit PCM Direct    2	 Play/Rcd AGC Kill 	Timer      X	 
;;   Ĵ
;;   Play Silence        3	     X    	X    	Timer      X	 
;;   Ĵ
;;   Play PCM Queue      4	 Compress  4/2 Bit 	Timer     Ref	 
;;   Ĵ
;;   Rcd 8 Bit Queue     5	    VOX   AGC Kill 	Timer      X	 
;;   Ĵ
;;   Rcd 4 Bit Queue     6	    VOX   AGC Kill 	Timer     Ref	 
;;   Ĵ
;;   Rcd 2 Bit Queue     7	    VOX   AGC Kill 	Timer     Ref	 
;;   Ĵ
;;   	 reserved         8	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         9	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         A	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         B	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         C	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         D	     X    	X    	  X        X	 
;;   Ĵ
;;   	 reserved         E	     X    	X    	  X        X	 
;;   Ĵ
;;   Manufacture Test    F	 		  Subcode		 
;;   
;;
;;	 System Control Commands:
;;
;;		 0001xxxxb  -  format of command:
;;
;;		     0000b  -  10h Terminate DAC/ADC mode, return to Idle.
;;		     0001b  -  11h Get ESP version
;;		     0010b  -  12h Get state variables
;;		     0011b  -  13h Enable Parallel Input
;;		     0100b  -  14h Disable Parallel Input
;;		     0101b  -  15h Test Parallel Input
;;		     0110b  -  16h undefined
;;		     0111b  -  17h undefined
;;		     1000b  -  18h undefined
;;		     1001b  -  19h undefined
;;		     1010b  -  1Ah undefined
;;		     1011b  -  1Bh undefined
;;		     1100b  -  1Ch undefined
;;		     1101b  -  1Dh undefined
;;		     1110b  -  1Eh undefined
;;		     1111b  -  1Fh undefined
;;
SYSCMDIDLE	equ	10h
SYSCMDGETID	equ	11h
SYSCMDGETSTT	equ	12h
SYSCMDENAPARR	equ	13h
SYSCMDDISPARR	equ	14h
SYSCMDTSTPARR	equ	15h
SYSCMDNA1	equ	16h
SYSCMDNA2	equ	17h
SYSCMDNA3	equ	18h
SYSCMDNA4	equ	19h
SYSCMDNA5	equ	1Ah
SYSCMDNA6	equ	1Bh
SYSCMDNA7	equ	1Ch
SYSCMDNA8	equ	1Dh
SYSCMDNA9	equ	1Eh
SYSCMDNAA	equ	1Fh


;;               0010xxxxb  -  format of command:
;;
;;		     0000b  -  20h 8 bit direct play.
;;		     0001b  -  21h ...
;;		     0010b  -  22h ...
;;		     0011b  -  23h ...
;;		     0100b  -  24h ...
;;		     0101b  -  25h ...
;;		     0110b  -  26h ...
;;		     0111b  -  27h ...
;;		     1000b  -  28h 8 bit direct record.
;;		     1001b  -  29h ...
;;		     1010b  -  2Ah ...
;;		     1011b  -  2Bh ...
;;		     1100b  -  2Ch ...
;;		     1101b  -  2Dh ...
;;		     1110b  -  2Eh ...
;;		     1111b  -  2Fh ...

SYSDIRPLAY	equ	20h
SYSDIRNA1	equ	21h
SYSDIRNA2	equ	22h
SYSDIRNA3	equ	23h
SYSDIRNA4	equ	24h
SYSDIRNA5	equ	25h
SYSDIRNA6	equ	26h
SYSDIRNA7	equ	27h
SYSDIRRECD	equ	28h
SYSDIRNA9	equ	29h
SYSDIRNAA	equ	2Ah
SYSDIRNAB	equ	2Bh
SYSDIRNAC	equ	2Ch
SYSDIRNAD	equ	2Dh
SYSDIRNAE	equ	2Eh
SYSDIRNAF	equ	2Fh

;;
;;	 Manufacturing Test Commands:
;;
;;		 1111xxxxb  -  format of command:
;;
;;		     0000b  -  F0h generate IRQ
;;		     0001b  -  F1h test fixed data
;;		     0010b  -  F2h test command port
;;		     0011b  -  F3h test data port
;;		     0100b  -  F4h magic code routine
;;		     0101b  -  F5h test low power
;;		     0110b  -  F6h power down
;;		     0111b  -  F7h checksum rom
;;		     1000b  -  F8h Input FIFO Test
;;		     1001b  -  F9h Output FIFO Test
;;		     1010b  -  FAh undefined
;;		     1011b  -  FBh undefined
;;		     1100b  -  FCh undefined
;;		     1101b  -  FDh undefined
;;		     1110b  -  FEh undefined
;;		     1111b  -  FFh undefined



